@@ -3,10 +3,7 @@
     celery.beat
     ~~~~~~~~~~~
 
-    The Celery periodic task scheduler.
-
-    :copyright: (c) 2009 - 2012 by Ask Solem.
-    :license: BSD, see LICENSE for more details.
+    The periodic task scheduler.
 
 """
 from __future__ import absolute_import
@@ -16,25 +13,28 @@ import os
 import time
 import shelve
 import sys
-import threading
 import traceback
-try:
-    import multiprocessing
-except ImportError:
-    multiprocessing = None  # noqa
 
+from billiard import Process, ensure_multiprocessing
 from kombu.utils import reprcall
+from kombu.utils.functional import maybe_promise
 
 from . import __version__
 from . import platforms
-from . import registry
 from . import signals
 from . import current_app
 from .app import app_or_default
-from .log import SilenceRepeated
 from .schedules import maybe_schedule, crontab
-from .utils import cached_property, instantiate, maybe_promise
+from .utils import cached_property
+from .utils.imports import instantiate
+from .utils.threads import Event, Thread
 from .utils.timeutils import humanize_seconds
+from .utils.log import get_logger
+
+logger = get_logger(__name__)
+debug, info, error = logger.debug, logger.info, logger.error
+
+DEFAULT_MAX_INTERVAL = 300  # 5 minutes
 
 
 class SchedulingError(Exception):
@@ -106,19 +106,19 @@ class ScheduleEntry(object):
         options).
 
         """
-        self.__dict__.update({"task": other.task, "schedule": other.schedule,
-                              "args": other.args, "kwargs": other.kwargs,
-                              "options": other.options})
+        self.__dict__.update({'task': other.task, 'schedule': other.schedule,
+                              'args': other.args, 'kwargs': other.kwargs,
+                              'options': other.options})
 
     def is_due(self):
-        """See :meth:`celery.task.base.PeriodicTask.is_due`."""
+        """See :meth:`~celery.schedule.schedule.is_due`."""
         return self.schedule.is_due(self.last_run_at)
 
     def __iter__(self):
         return vars(self).iteritems()
 
     def __repr__(self):
-        return ("<Entry: %s %s {%s}" % (self.name,
+        return ('<Entry: %s %s {%s}' % (self.name,
                     reprcall(self.task, self.args or (), self.kwargs or {}),
                     self.schedule))
 
@@ -127,7 +127,6 @@ class Scheduler(object):
     """Scheduler for periodic tasks.
 
     :keyword schedule: see :attr:`schedule`.
-    :keyword logger: see :attr:`logger`.
     :keyword max_interval: see :attr:`max_interval`.
 
     """
@@ -137,52 +136,49 @@ class Scheduler(object):
     #: The schedule dict/shelve.
     schedule = None
 
-    #: Current logger.
-    logger = None
-
     #: Maximum time to sleep between re-checking the schedule.
-    max_interval = 1
+    max_interval = DEFAULT_MAX_INTERVAL
 
     #: How often to sync the schedule (3 minutes by default)
     sync_every = 3 * 60
 
     _last_sync = None
 
-    def __init__(self, schedule=None, logger=None, max_interval=None,
+    logger = logger  # compat
+
+    def __init__(self, schedule=None, max_interval=None,
             app=None, Publisher=None, lazy=False, **kwargs):
         app = self.app = app_or_default(app)
         self.data = maybe_promise({} if schedule is None else schedule)
-        self.logger = logger or app.log.get_default_logger(name="celery.beat")
-        self.max_interval = max_interval or \
-                                app.conf.CELERYBEAT_MAX_LOOP_INTERVAL
-        self.Publisher = Publisher or app.amqp.TaskPublisher
+        self.max_interval = (max_interval
+                                or app.conf.CELERYBEAT_MAX_LOOP_INTERVAL
+                                or self.max_interval)
+        self.Publisher = Publisher or app.amqp.TaskProducer
         if not lazy:
             self.setup_schedule()
 
     def install_default_entries(self, data):
         entries = {}
         if self.app.conf.CELERY_TASK_RESULT_EXPIRES:
-            if "celery.backend_cleanup" not in data:
-                entries["celery.backend_cleanup"] = {
-                        "task": "celery.backend_cleanup",
-                        "schedule": crontab("0", "4", "*"),
-                        "options": {"expires": 12 * 3600}}
+            if 'celery.backend_cleanup' not in data:
+                entries['celery.backend_cleanup'] = {
+                        'task': 'celery.backend_cleanup',
+                        'schedule': crontab('0', '4', '*'),
+                        'options': {'expires': 12 * 3600}}
         self.update_from_dict(entries)
 
     def maybe_due(self, entry, publisher=None):
         is_due, next_time_to_run = entry.is_due()
 
         if is_due:
-            self.logger.info("Scheduler: Sending due task %s", entry.task)
+            info('Scheduler: Sending due task %s', entry.task)
             try:
                 result = self.apply_async(entry, publisher=publisher)
             except Exception, exc:
-                self.logger.error("Message Error: %s\n%s", exc,
-                                  traceback.format_stack(),
-                                  exc_info=True)
+                error('Message Error: %s\n%s',
+                      exc, traceback.format_stack(), exc_info=True)
             else:
-                self.logger.debug("%s sent. id->%s", entry.task,
-                                                     result.task_id)
+                debug('%s sent. id->%s', entry.task, result.id)
         return next_time_to_run
 
     def tick(self):
@@ -215,7 +211,7 @@ class Scheduler(object):
         # so we have that done if an exception is raised (doesn't schedule
         # forever.)
         entry = self.reserve(entry)
-        task = registry.tasks.get(entry.task)
+        task = self.app.tasks.get(entry.task)
 
         try:
             if task:
@@ -230,12 +226,12 @@ class Scheduler(object):
             raise SchedulingError, SchedulingError(
                 "Couldn't apply scheduled task %s: %s" % (
                     entry.name, exc)), sys.exc_info()[2]
-
-        if self.should_sync():
-            self._do_sync()
+        finally:
+            if self.should_sync():
+                self._do_sync()
         return result
 
-    def send_task(self, *args, **kwargs):               # pragma: no cover
+    def send_task(self, *args, **kwargs):
         return self.app.send_task(*args, **kwargs)
 
     def setup_schedule(self):
@@ -243,7 +239,7 @@ class Scheduler(object):
 
     def _do_sync(self):
         try:
-            self.logger.debug("Celerybeat: Synchronizing schedule...")
+            debug('Celerybeat: Synchronizing schedule...')
             self.sync()
         finally:
             self._last_sync = time.time()
@@ -284,50 +280,48 @@ class Scheduler(object):
             else:
                 schedule[key] = entry
 
+    def _ensure_connected(self):
+        # callback called for each retry while the connection
+        # can't be established.
+        def _error_handler(exc, interval):
+            error('Celerybeat: Connection error: %s. '
+                  'Trying again in %s seconds...', exc, interval)
+
+        return self.connection.ensure_connection(_error_handler,
+                    self.app.conf.BROKER_CONNECTION_MAX_RETRIES)
+
     def get_schedule(self):
         return self.data
 
     def set_schedule(self, schedule):
         self.data = schedule
-
-    def _ensure_connected(self):
-        # callback called for each retry while the connection
-        # can't be established.
-        def _error_handler(exc, interval):
-            self.logger.error("Celerybeat: Connection error: %s. "
-                              "Trying again in %s seconds...", exc, interval)
-
-        return self.connection.ensure_connection(_error_handler,
-                    self.app.conf.BROKER_CONNECTION_MAX_RETRIES)
+    schedule = property(get_schedule, set_schedule)
 
     @cached_property
     def connection(self):
-        return self.app.broker_connection()
+        return self.app.connection()
 
     @cached_property
     def publisher(self):
-        return self.Publisher(connection=self._ensure_connected())
-
-    @property
-    def schedule(self):
-        return self.get_schedule()
+        return self.Publisher(self._ensure_connected())
 
     @property
     def info(self):
-        return ""
+        return ''
 
 
 class PersistentScheduler(Scheduler):
     persistence = shelve
+    known_suffixes = ('', '.db', '.dat', '.bak', '.dir')
 
     _store = None
 
     def __init__(self, *args, **kwargs):
-        self.schedule_filename = kwargs.get("schedule_filename")
+        self.schedule_filename = kwargs.get('schedule_filename')
         Scheduler.__init__(self, *args, **kwargs)
 
     def _remove_db(self):
-        for suffix in "", ".db", ".dat", ".bak", ".dir":
+        for suffix in self.known_suffixes:
             try:
                 os.remove(self.schedule_filename + suffix)
             except OSError, exc:
@@ -338,27 +332,30 @@ class PersistentScheduler(Scheduler):
         try:
             self._store = self.persistence.open(self.schedule_filename,
                                                 writeback=True)
-            entries = self._store.setdefault("entries", {})
+            entries = self._store.setdefault('entries', {})
         except Exception, exc:
-            self.logger.error("Removing corrupted schedule file %r: %r",
-                              self.schedule_filename, exc, exc_info=True)
+            error('Removing corrupted schedule file %r: %r',
+                  self.schedule_filename, exc, exc_info=True)
             self._remove_db()
             self._store = self.persistence.open(self.schedule_filename,
                                                 writeback=True)
         else:
-            if "__version__" not in self._store:
+            if '__version__' not in self._store:
                 self._store.clear()   # remove schedule at 2.2.2 upgrade.
-        entries = self._store.setdefault("entries", {})
+        entries = self._store.setdefault('entries', {})
         self.merge_inplace(self.app.conf.CELERYBEAT_SCHEDULE)
         self.install_default_entries(self.schedule)
-        self._store["__version__"] = __version__
+        self._store['__version__'] = __version__
         self.sync()
-        self.logger.debug("Current schedule:\n" +
-                          "\n".join(repr(entry)
+        debug('Current schedule:\n' + '\n'.join(repr(entry)
                                     for entry in entries.itervalues()))
 
     def get_schedule(self):
-        return self._store["entries"]
+        return self._store['entries']
+
+    def set_schedule(self, schedule):
+        self._store['entries'] = schedule
+    schedule = property(get_schedule, set_schedule)
 
     def sync(self):
         if self._store is not None:
@@ -370,42 +367,39 @@ class PersistentScheduler(Scheduler):
 
     @property
     def info(self):
-        return "    . db -> %s" % (self.schedule_filename, )
+        return '    . db -> %s' % (self.schedule_filename, )
 
 
 class Service(object):
     scheduler_cls = PersistentScheduler
 
-    def __init__(self, logger=None, max_interval=None, schedule_filename=None,
+    def __init__(self, max_interval=None, schedule_filename=None,
             scheduler_cls=None, app=None):
         app = self.app = app_or_default(app)
-        self.max_interval = max_interval or \
-                                app.conf.CELERYBEAT_MAX_LOOP_INTERVAL
+        self.max_interval = (max_interval
+                             or app.conf.CELERYBEAT_MAX_LOOP_INTERVAL)
         self.scheduler_cls = scheduler_cls or self.scheduler_cls
-        self.logger = logger or app.log.get_default_logger(name="celery.beat")
         self.schedule_filename = schedule_filename or \
                                     app.conf.CELERYBEAT_SCHEDULE_FILENAME
 
-        self._is_shutdown = threading.Event()
-        self._is_stopped = threading.Event()
-        self.debug = SilenceRepeated(self.logger.debug,
-                        10 if self.max_interval < 60 else 1)
+        self._is_shutdown = Event()
+        self._is_stopped = Event()
 
     def start(self, embedded_process=False):
-        self.logger.info("Celerybeat: Starting...")
-        self.logger.debug("Celerybeat: Ticking with max interval->%s",
-                          humanize_seconds(self.scheduler.max_interval))
+        info('Celerybeat: Starting...')
+        debug('Celerybeat: Ticking with max interval->%s',
+              humanize_seconds(self.scheduler.max_interval))
 
         signals.beat_init.send(sender=self)
         if embedded_process:
             signals.beat_embedded_init.send(sender=self)
-            platforms.set_process_title("celerybeat")
+            platforms.set_process_title('celerybeat')
 
         try:
-            while not self._is_shutdown.isSet():
+            while not self._is_shutdown.is_set():
                 interval = self.scheduler.tick()
-                self.debug("Celerybeat: Waking up %s.",
-                           humanize_seconds(interval, prefix="in "))
+                debug('Celerybeat: Waking up %s.',
+                      humanize_seconds(interval, prefix='in '))
                 time.sleep(interval)
         except (KeyboardInterrupt, SystemExit):
             self._is_shutdown.set()
@@ -417,7 +411,7 @@ class Service(object):
         self._is_stopped.set()
 
     def stop(self, wait=False):
-        self.logger.info("Celerybeat: Shutting down...")
+        info('Celerybeat: Shutting down...')
         self._is_shutdown.set()
         wait and self._is_stopped.wait()  # block until shutdown done.
 
@@ -426,7 +420,6 @@ class Service(object):
         scheduler = instantiate(self.scheduler_cls,
                                 app=self.app,
                                 schedule_filename=filename,
-                                logger=self.logger,
                                 max_interval=self.max_interval,
                                 lazy=lazy)
         return scheduler
@@ -436,14 +429,14 @@ class Service(object):
         return self.get_scheduler()
 
 
-class _Threaded(threading.Thread):
+class _Threaded(Thread):
     """Embedded task scheduler using threading."""
 
     def __init__(self, *args, **kwargs):
         super(_Threaded, self).__init__()
         self.service = Service(*args, **kwargs)
-        self.setDaemon(True)
-        self.setName("Beat")
+        self.daemon = True
+        self.name = 'Beat'
 
     def run(self):
         self.service.start()
@@ -452,25 +445,25 @@ class _Threaded(threading.Thread):
         self.service.stop(wait=True)
 
 
-if multiprocessing is not None:
-
-    class _Process(multiprocessing.Process):
-        """Embedded task scheduler using multiprocessing."""
+try:
+    ensure_multiprocessing()
+except NotImplementedError:     # pragma: no cover
+    _Process = None
+else:
+    class _Process(Process):    # noqa
 
         def __init__(self, *args, **kwargs):
             super(_Process, self).__init__()
             self.service = Service(*args, **kwargs)
-            self.name = "Beat"
+            self.name = 'Beat'
 
         def run(self):
-            platforms.signals.reset("SIGTERM")
+            platforms.signals.reset('SIGTERM')
             self.service.start(embedded_process=True)
 
         def stop(self):
             self.service.stop()
             self.terminate()
-else:
-    _Process = None
 
 
 def EmbeddedService(*args, **kwargs):
@@ -480,10 +473,9 @@ def EmbeddedService(*args, **kwargs):
         Default is :const:`False`.
 
     """
-    if kwargs.pop("thread", False) or _Process is None:
+    if kwargs.pop('thread', False) or _Process is None:
         # Need short max interval to be able to stop thread
         # in reasonable time.
-        kwargs.setdefault("max_interval", 1)
+        kwargs.setdefault('max_interval', 1)
         return _Threaded(*args, **kwargs)
-
     return _Process(*args, **kwargs)
