@@ -7,9 +7,6 @@ This module contains the component responsible for consuming messages
 from the broker, processing the messages and keeping the broker connections
 up and running.
 
-:copyright: (c) 2009 - 2012 by Ask Solem.
-:license: BSD, see LICENSE for more details.
-
 
 * :meth:`~Consumer.start` is an infinite loop, which only iterates
   again if the connection is lost. For each iteration (at start, or if the
@@ -46,7 +43,7 @@ up and running.
   are acknowledged immediately and logged, so the message is not resent
   again, and again.
 
-* If the task has an ETA/countdown, the task is moved to the `eta_schedule`
+* If the task has an ETA/countdown, the task is moved to the `timer`
   so the :class:`timer2.Timer` can schedule it at its
   deadline. Tasks without an eta are moved immediately to the `ready_queue`,
   so they can be picked up by the :class:`~celery.worker.mediator.Mediator`
@@ -79,17 +76,24 @@ from __future__ import with_statement
 import logging
 import socket
 import threading
-import warnings
 
-from ..abstract import StartStopComponent
-from ..app import app_or_default
-from ..datastructures import AttributeDict
-from ..exceptions import InvalidTaskError
-from ..registry import tasks
-from ..utils import noop
-from ..utils import timer2
-from ..utils.encoding import safe_repr
+from time import sleep
+from Queue import Empty
+
+from kombu.utils.encoding import safe_repr
+from kombu.utils.eventio import READ, WRITE, ERR
+
+from celery.app import app_or_default
+from celery.datastructures import AttributeDict
+from celery.exceptions import InvalidTaskError, SystemTerminate
+from celery.task.trace import build_tracer
+from celery.utils import timer2
+from celery.utils.functional import noop
+from celery.utils.log import get_logger
+from celery.utils import text
+
 from . import state
+from .bootsteps import StartStopComponent
 from .control import Panel
 from .heartbeat import Heart
 
@@ -99,6 +103,11 @@ CLOSE = 0x2
 #: Prefetch count can't exceed short.
 PREFETCH_COUNT_MAX = 0xFFFF
 
+UNKNOWN_FORMAT = """\
+Received and deleted unknown message. Wrong destination?!?
+
+The full contents of the message body was: %s
+"""
 #: Error message for when an unregistered task is received.
 UNKNOWN_TASK_ERROR = """\
 Received unregistered task of type %s.
@@ -129,65 +138,66 @@ body: %s {content_type:%s content_encoding:%s delivery_info:%s}\
 """
 
 
+RETRY_CONNECTION = """\
+Consumer: Connection to broker lost. \
+Trying to re-establish the connection...\
+"""
+
+task_reserved = state.task_reserved
+
+logger = get_logger(__name__)
+info, warn, error, crit = (logger.info, logger.warn,
+                           logger.error, logger.critical)
+
+
+def debug(msg, *args, **kwargs):
+    logger.debug('Consumer: %s' % (msg, ), *args, **kwargs)
+
+
+def dump_body(m, body):
+    return "%s (%sb)" % (text.truncate(safe_repr(body), 1024), len(m.body))
+
+
 class Component(StartStopComponent):
-    name = "worker.consumer"
+    name = 'worker.consumer'
     last = True
 
+    def Consumer(self, w):
+        return (w.consumer_cls or
+                Consumer if w.hub else BlockingConsumer)
+
     def create(self, w):
         prefetch_count = w.concurrency * w.prefetch_multiplier
-        c = w.consumer = self.instantiate(
-                w.consumer_cls, w.ready_queue, w.scheduler,
-                logger=w.logger, hostname=w.hostname,
+        c = w.consumer = self.instantiate(self.Consumer(w),
+                w.ready_queue,
+                hostname=w.hostname,
                 send_events=w.send_events,
                 init_callback=w.ready_callback,
                 initial_prefetch_count=prefetch_count,
                 pool=w.pool,
-                priority_timer=w.priority_timer,
+                timer=w.timer,
                 app=w.app,
-                controller=w)
+                controller=w,
+                hub=w.hub)
         return c
 
 
 class QoS(object):
-    """Quality of Service for Channel.
-
-    For thread-safe increment/decrement of a channels prefetch count value.
+    """Thread safe increment/decrement of a channels prefetch_count.
 
     :param consumer: A :class:`kombu.messaging.Consumer` instance.
     :param initial_value: Initial prefetch count value.
-    :param logger: Logger used to log debug messages.
 
     """
     prev = None
 
-    def __init__(self, consumer, initial_value, logger):
+    def __init__(self, consumer, initial_value):
         self.consumer = consumer
-        self.logger = logger
         self._mutex = threading.RLock()
-        self.value = initial_value
+        self.value = initial_value or 0
 
-    def increment(self, n=1):
-        """Increment the current prefetch count value by n."""
-        with self._mutex:
-            if self.value:
-                new_value = self.value + max(n, 0)
-                self.value = self.set(new_value)
-        return self.value
-
-    def _sub(self, n=1):
-        assert self.value - n > 1
-        self.value -= n
-
-    def decrement(self, n=1):
-        """Decrement the current prefetch count value by n."""
-        with self._mutex:
-            if self.value:
-                self._sub(n)
-                self.set(self.value)
-        return self.value
-
-    def decrement_eventually(self, n=1):
-        """Decrement the value, but do not update the qos.
+    def increment_eventually(self, n=1):
+        """Increment the value, but do not update the channels QoS.
 
         The MainThread will be responsible for calling :meth:`update`
         when necessary.
@@ -195,17 +205,30 @@ class QoS(object):
         """
         with self._mutex:
             if self.value:
-                self._sub(n)
+                self.value = self.value + max(n, 0)
+        return self.value
+
+    def decrement_eventually(self, n=1):
+        """Decrement the value, but do not update the channels QoS.
+
+        The MainThread will be responsible for calling :meth:`update`
+        when necessary.
+
+        """
+        with self._mutex:
+            if self.value:
+                self.value -= n
+        return self.value
 
     def set(self, pcount):
         """Set channel prefetch_count setting."""
         if pcount != self.prev:
             new_value = pcount
             if pcount > PREFETCH_COUNT_MAX:
-                self.logger.warning("QoS: Disabled: prefetch_count exceeds %r",
-                                    PREFETCH_COUNT_MAX)
+                warn('QoS: Disabled: prefetch_count exceeds %r',
+                     PREFETCH_COUNT_MAX)
                 new_value = 0
-            self.logger.debug("basic.qos: prefetch_count->%s", new_value)
+            debug('basic.qos: prefetch_count->%s', new_value)
             self.consumer.qos(prefetch_count=new_value)
             self.prev = pcount
         return pcount
@@ -221,16 +244,13 @@ class Consumer(object):
     move them to the ready queue for task processing.
 
     :param ready_queue: See :attr:`ready_queue`.
-    :param eta_schedule: See :attr:`eta_schedule`.
+    :param timer: See :attr:`timer`.
 
     """
 
     #: The queue that holds tasks ready for immediate processing.
     ready_queue = None
 
-    #: Timer for tasks with an ETA/countdown.
-    eta_schedule = None
-
     #: Enable/disable events.
     send_events = False
 
@@ -253,9 +273,6 @@ class Consumer(object):
     #: went offline/disappeared.
     heart = None
 
-    #: The logger instance to use.  Defaults to the default Celery logger.
-    logger = None
-
     #: The broker connection.
     connection = None
 
@@ -275,50 +292,55 @@ class Consumer(object):
 
     #: A timer used for high-priority internal tasks, such
     #: as sending heartbeats.
-    priority_timer = None
+    timer = None
 
     # Consumer state, can be RUN or CLOSE.
     _state = None
 
-    def __init__(self, ready_queue, eta_schedule, logger,
+    def __init__(self, ready_queue,
             init_callback=noop, send_events=False, hostname=None,
             initial_prefetch_count=2, pool=None, app=None,
-            priority_timer=None, controller=None):
+            timer=None, controller=None, hub=None, **kwargs):
         self.app = app_or_default(app)
         self.connection = None
         self.task_consumer = None
         self.controller = controller
         self.broadcast_consumer = None
         self.ready_queue = ready_queue
-        self.eta_schedule = eta_schedule
         self.send_events = send_events
         self.init_callback = init_callback
-        self.logger = logger
         self.hostname = hostname or socket.gethostname()
         self.initial_prefetch_count = initial_prefetch_count
         self.event_dispatcher = None
         self.heart = None
         self.pool = pool
-        self.priority_timer = priority_timer or timer2.default_timer
+        self.timer = timer or timer2.default_timer
         pidbox_state = AttributeDict(app=self.app,
-                                     logger=logger,
                                      hostname=self.hostname,
                                      listener=self,     # pre 2.2
                                      consumer=self)
         self.pidbox_node = self.app.control.mailbox.Node(self.hostname,
                                                          state=pidbox_state,
                                                          handlers=Panel.data)
-        conninfo = self.app.broker_connection()
+        conninfo = self.app.connection()
         self.connection_errors = conninfo.connection_errors
         self.channel_errors = conninfo.channel_errors
 
-        self._does_info = self.logger.isEnabledFor(logging.INFO)
+        self._does_info = logger.isEnabledFor(logging.INFO)
         self.strategies = {}
+        if hub:
+            hub.on_init.append(self.on_poll_init)
+        self.hub = hub
+        self._quick_put = self.ready_queue.put
 
     def update_strategies(self):
         S = self.strategies
-        for task in tasks.itervalues():
-            S[task.name] = task.start_strategy(self.app, self)
+        app = self.app
+        loader = app.loader
+        hostname = self.hostname
+        for name, task in self.app.tasks.iteritems():
+            S[name] = task.start_strategy(app, self)
+            task.__trace__ = build_tracer(name, task, loader, hostname)
 
     def start(self):
         """Start the consumer.
@@ -332,50 +354,120 @@ class Consumer(object):
         self.init_callback(self)
 
         while self._state != CLOSE:
+            self.maybe_shutdown()
             try:
                 self.reset_connection()
                 self.consume_messages()
             except self.connection_errors + self.channel_errors:
-                self.logger.error("Consumer: Connection to broker lost."
-                                + " Trying to re-establish the connection...",
-                                exc_info=True)
+                error(RETRY_CONNECTION, exc_info=True)
 
-    def consume_messages(self):
+    def on_poll_init(self, hub):
+        hub.update_readers(self.connection.eventmap)
+        self.connection.transport.on_poll_init(hub.poller)
+
+    def consume_messages(self, sleep=sleep, min=min, Empty=Empty):
         """Consume messages forever (or until an exception is raised)."""
-        self._debug("Starting message consumer...")
-        self.task_consumer.consume()
-        self._debug("Ready to accept tasks!")
 
-        while self._state != CLOSE and self.connection:
-            if self.qos.prev != self.qos.value:
-                self.qos.update()
-            try:
-                self.connection.drain_events(timeout=1)
-            except socket.timeout:
-                pass
-            except socket.error:
-                if self._state != CLOSE:
-                    raise
+        with self.hub as hub:
+            qos = self.qos
+            update_qos = qos.update
+            update_readers = hub.update_readers
+            readers, writers = hub.readers, hub.writers
+            poll = hub.poller.poll
+            fire_timers = hub.fire_timers
+            scheduled = hub.timer._queue
+            connection = self.connection
+            on_poll_start = connection.transport.on_poll_start
+            strategies = self.strategies
+            drain_nowait = connection.drain_nowait
+            on_task_callbacks = hub.on_task
+            keep_draining = connection.transport.nb_keep_draining
 
-    def on_task(self, task):
+            def on_task_received(body, message):
+                if on_task_callbacks:
+                    [callback() for callback in on_task_callbacks]
+                try:
+                    name = body['task']
+                except (KeyError, TypeError):
+                    return self.handle_unknown_message(body, message)
+                try:
+                    strategies[name](message, body, message.ack_log_error)
+                except KeyError, exc:
+                    self.handle_unknown_task(body, message, exc)
+                except InvalidTaskError, exc:
+                    self.handle_invalid_task(body, message, exc)
+                #fire_timers()
+
+            self.task_consumer.callbacks = [on_task_received]
+            self.task_consumer.consume()
+
+            debug('Ready to accept tasks!')
+
+            while self._state != CLOSE and self.connection:
+                # shutdown if signal handlers told us to.
+                if state.should_stop:
+                    raise SystemExit()
+                elif state.should_terminate:
+                    raise SystemTerminate()
+
+                # fire any ready timers, this also returns
+                # the number of seconds until we need to fire timers again.
+                poll_timeout = fire_timers() if scheduled else 1
+
+                # We only update QoS when there is no more messages to read.
+                # This groups together qos calls, and makes sure that remote
+                # control commands will be prioritized over task messages.
+                if qos.prev != qos.value:
+                    update_qos()
+
+                update_readers(on_poll_start())
+                if readers or writers:
+                    connection.more_to_read = True
+                    while connection.more_to_read:
+                        for fileno, event in poll(poll_timeout) or ():
+                            try:
+                                if event & READ:
+                                    readers[fileno](fileno, event)
+                                if event & WRITE:
+                                    writers[fileno](fileno, event)
+                                if event & ERR:
+                                    for handlermap in readers, writers:
+                                        try:
+                                            handlermap[fileno](fileno, event)
+                                        except KeyError:
+                                            pass
+                            except Empty:
+                                continue
+                            except socket.error:
+                                if self._state != CLOSE:  # pragma: no cover
+                                    raise
+                        if keep_draining:
+                            drain_nowait()
+                            poll_timeout = 0
+                        else:
+                            connection.more_to_read = False
+                else:
+                    # no sockets yet, startup is probably not done.
+                    sleep(min(poll_timeout, 0.1))
+
+    def on_task(self, task, task_reserved=task_reserved):
         """Handle received task.
 
         If the task has an `eta` we enter it into the ETA schedule,
         otherwise we move it the ready queue for immediate processing.
 
         """
-
         if task.revoked():
             return
 
         if self._does_info:
-            self.logger.info("Got task from broker: %s", task.shortinfo())
+            info('Got task from broker: %s', task)
 
         if self.event_dispatcher.enabled:
-            self.event_dispatcher.send("task-received", uuid=task.task_id,
-                    name=task.task_name, args=safe_repr(task.args),
+            self.event_dispatcher.send('task-received', uuid=task.id,
+                    name=task.name, args=safe_repr(task.args),
                     kwargs=safe_repr(task.kwargs),
-                    retries=task.request_dict.get("retries", 0),
+                    retries=task.request_dict.get('retries', 0),
                     eta=task.eta and task.eta.isoformat(),
                     expires=task.expires and task.expires.isoformat())
 
@@ -383,44 +475,52 @@ class Consumer(object):
             try:
                 eta = timer2.to_timestamp(task.eta)
             except OverflowError, exc:
-                self.logger.error(
-                    "Couldn't convert eta %s to timestamp: %r. Task: %r",
-                    task.eta, exc, task.info(safe=True),
-                    exc_info=True)
+                error("Couldn't convert eta %s to timestamp: %r. Task: %r",
+                      task.eta, exc, task.info(safe=True), exc_info=True)
                 task.acknowledge()
             else:
-                self.qos.increment()
-                self.eta_schedule.apply_at(eta,
-                                           self.apply_eta_task, (task, ))
+                self.qos.increment_eventually()
+                self.timer.apply_at(eta, self.apply_eta_task, (task, ),
+                                    priority=6)
         else:
-            state.task_reserved(task)
-            self.ready_queue.put(task)
+            task_reserved(task)
+            self._quick_put(task)
 
     def on_control(self, body, message):
         """Process remote control command message."""
         try:
             self.pidbox_node.handle_message(body, message)
         except KeyError, exc:
-            self.logger.error("No such control command: %s", exc)
+            error('No such control command: %s', exc)
         except Exception, exc:
-            self.logger.error(
-                "Error occurred while handling control command: %r",
-                    exc, exc_info=True)
+            error('Control command error: %r', exc, exc_info=True)
             self.reset_pidbox_node()
 
     def apply_eta_task(self, task):
         """Method called by the timer to apply a task with an
         ETA/countdown."""
-        state.task_reserved(task)
-        self.ready_queue.put(task)
+        task_reserved(task)
+        self._quick_put(task)
         self.qos.decrement_eventually()
 
     def _message_report(self, body, message):
-        return MESSAGE_REPORT_FMT % (safe_repr(body),
+        return MESSAGE_REPORT_FMT % (dump_body(message, body),
                                      safe_repr(message.content_type),
                                      safe_repr(message.content_encoding),
                                      safe_repr(message.delivery_info))
 
+    def handle_unknown_message(self, body, message):
+        warn(UNKNOWN_FORMAT, self._message_report(body, message))
+        message.reject_log_error(logger, self.connection_errors)
+
+    def handle_unknown_task(self, body, message, exc):
+        error(UNKNOWN_TASK_ERROR, exc, dump_body(message, body), exc_info=True)
+        message.reject_log_error(logger, self.connection_errors)
+
+    def handle_invalid_task(self, body, message, exc):
+        error(INVALID_TASK_ERROR, exc, dump_body(message, body), exc_info=True)
+        message.reject_log_error(logger, self.connection_errors)
+
     def receive_message(self, body, message):
         """Handles incoming messages.
 
@@ -429,25 +529,16 @@ class Consumer(object):
 
         """
         try:
-            name = body["task"]
+            name = body['task']
         except (KeyError, TypeError):
-            warnings.warn(RuntimeWarning(
-                "Received and deleted unknown message. Wrong destination?!? \
-                the full contents of the message body was: %s" % (
-                 self._message_report(body, message), )))
-            message.reject_log_error(self.logger, self.connection_errors)
-            return
+            return self.handle_unknown_message(body, message)
 
         try:
             self.strategies[name](message, body, message.ack_log_error)
         except KeyError, exc:
-            self.logger.error(UNKNOWN_TASK_ERROR, exc, safe_repr(body),
-                              exc_info=True)
-            message.reject_log_error(self.logger, self.connection_errors)
+            self.handle_unknown_task(body, message, exc)
         except InvalidTaskError, exc:
-            self.logger.error(INVALID_TASK_ERROR, str(exc), safe_repr(body),
-                              exc_info=True)
-            message.reject_log_error(self.logger, self.connection_errors)
+            self.handle_invalid_task(body, message, exc)
 
     def maybe_conn_error(self, fun):
         """Applies function but ignores any connection or channel
@@ -467,14 +558,14 @@ class Consumer(object):
         connection, self.connection = self.connection, None
 
         if self.task_consumer:
-            self._debug("Closing consumer channel...")
+            debug('Closing consumer channel...')
             self.task_consumer = \
                     self.maybe_conn_error(self.task_consumer.close)
 
         self.stop_pidbox_node()
 
         if connection:
-            self._debug("Closing broker connection...")
+            debug('Closing broker connection...')
             self.maybe_conn_error(connection.close)
 
     def stop_consumers(self, close_connection=True):
@@ -490,19 +581,19 @@ class Consumer(object):
 
         if self.heart:
             # Stop the heartbeat thread if it's running.
-            self.logger.debug("Heart: Going into cardiac arrest...")
+            debug('Heart: Going into cardiac arrest...')
             self.heart = self.heart.stop()
 
-        self._debug("Cancelling task consumer...")
+        debug('Cancelling task consumer...')
         if self.task_consumer:
             self.maybe_conn_error(self.task_consumer.cancel)
 
         if self.event_dispatcher:
-            self._debug("Shutting down event dispatcher...")
+            debug('Shutting down event dispatcher...')
             self.event_dispatcher = \
                     self.maybe_conn_error(self.event_dispatcher.close)
 
-        self._debug("Cancelling broadcast consumer...")
+        debug('Cancelling broadcast consumer...')
         if self.broadcast_consumer:
             self.maybe_conn_error(self.broadcast_consumer.cancel)
 
@@ -520,10 +611,9 @@ class Consumer(object):
         :param exc: The original exception instance.
 
         """
-        self.logger.critical(
-            "Can't decode message body: %r (type:%r encoding:%r raw:%r')",
-                    exc, message.content_type, message.content_encoding,
-                    safe_repr(message.body))
+        crit("Can't decode message body: %r (type:%r encoding:%r raw:%r')",
+             exc, message.content_type, message.content_encoding,
+             dump_body(message, message.body))
         message.ack()
 
     def reset_pidbox_node(self):
@@ -541,16 +631,15 @@ class Consumer(object):
         self.pidbox_node.channel = self.connection.channel()
         self.broadcast_consumer = self.pidbox_node.listen(
                                         callback=self.on_control)
-        self.broadcast_consumer.consume()
 
     def stop_pidbox_node(self):
         if self._pidbox_node_stopped:
             self._pidbox_node_shutdown.set()
-            self._debug("Waiting for broadcast thread to shutdown...")
+            debug('Waiting for broadcast thread to shutdown...')
             self._pidbox_node_stopped.wait()
             self._pidbox_node_stopped = self._pidbox_node_shutdown = None
         elif self.broadcast_consumer:
-            self._debug("Closing broadcast channel...")
+            debug('Closing broadcast channel...')
             self.broadcast_consumer = \
                 self.maybe_conn_error(self.broadcast_consumer.channel.close)
 
@@ -558,7 +647,7 @@ class Consumer(object):
         """Sets up the process mailbox when running in a greenlet
         environment."""
         # THIS CODE IS TERRIBLE
-        # Luckily work has already started rewriting the Consumer for 3.0.
+        # Luckily work has already started rewriting the Consumer for 4.0.
         self._pidbox_node_shutdown = threading.Event()
         self._pidbox_node_stopped = threading.Event()
         try:
@@ -578,28 +667,24 @@ class Consumer(object):
     def reset_connection(self):
         """Re-establish the broker connection and set up consumers,
         heartbeat and the event dispatcher."""
-        self._debug("Re-establishing connection to the broker...")
+        debug('Re-establishing connection to the broker...')
         self.stop_consumers()
 
         # Clear internal queues to get rid of old messages.
         # They can't be acked anyway, as a delivery tag is specific
         # to the current channel.
         self.ready_queue.clear()
-        self.eta_schedule.clear()
+        self.timer.clear()
 
         # Re-establish the broker connection and setup the task consumer.
         self.connection = self._open_connection()
-        self._debug("Connection established.")
-        self.task_consumer = self.app.amqp.get_task_consumer(self.connection,
+        debug('Connection established.')
+        self.task_consumer = self.app.amqp.TaskConsumer(self.connection,
                                     on_decode_error=self.on_decode_error)
         # QoS: Reset prefetch window.
-        self.qos = QoS(self.task_consumer,
-                       self.initial_prefetch_count, self.logger)
+        self.qos = QoS(self.task_consumer, self.initial_prefetch_count)
         self.qos.update()
 
-        # receive_message handles incoming messages.
-        self.task_consumer.register_callback(self.receive_message)
-
         # Setup the process mailbox.
         self.reset_pidbox_node()
 
@@ -628,7 +713,7 @@ class Consumer(object):
         can tell if the worker is off-line/missing.
 
         """
-        self.heart = Heart(self.priority_timer, self.event_dispatcher)
+        self.heart = Heart(self.timer, self.event_dispatcher)
         self.heart.start()
 
     def _open_connection(self):
@@ -642,19 +727,20 @@ class Consumer(object):
         # Callback called for each retry while the connection
         # can't be established.
         def _error_handler(exc, interval):
-            self.logger.error("Consumer: Connection Error: %s. "
-                              "Trying again in %d seconds...", exc, interval)
+            error('Consumer: Connection Error: %s. '
+                  'Trying again in %d seconds...', exc, interval)
 
         # remember that the connection is lazy, it won't establish
         # until it's needed.
-        conn = self.app.broker_connection()
+        conn = self.app.connection()
         if not self.app.conf.BROKER_CONNECTION_RETRY:
             # retry disabled, just call connect directly.
             conn.connect()
             return conn
 
         return conn.ensure_connection(_error_handler,
-                    self.app.conf.BROKER_CONNECTION_MAX_RETRIES)
+                    self.app.conf.BROKER_CONNECTION_MAX_RETRIES,
+                    callback=self.maybe_shutdown)
 
     def stop(self):
         """Stop consuming.
@@ -665,10 +751,19 @@ class Consumer(object):
         """
         # Notifies other threads that this instance can't be used
         # anymore.
-        self._state = CLOSE
-        self._debug("Stopping consumers...")
+        self.close()
+        debug('Stopping consumers...')
         self.stop_consumers(close_connection=False)
 
+    def close(self):
+        self._state = CLOSE
+
+    def maybe_shutdown(self):
+        if state.should_stop:
+            raise SystemExit()
+        elif state.should_terminate:
+            raise SystemTerminate()
+
     @property
     def info(self):
         """Returns information about this consumer instance
@@ -681,8 +776,27 @@ class Consumer(object):
         conninfo = {}
         if self.connection:
             conninfo = self.connection.info()
-            conninfo.pop("password", None)  # don't send password.
-        return {"broker": conninfo, "prefetch_count": self.qos.value}
+            conninfo.pop('password', None)  # don't send password.
+        return {'broker': conninfo, 'prefetch_count': self.qos.value}
 
-    def _debug(self, msg, **kwargs):
-        self.logger.debug("Consumer: %s", msg, **kwargs)
+
+class BlockingConsumer(Consumer):
+
+    def consume_messages(self):
+        # receive_message handles incoming messages.
+        self.task_consumer.register_callback(self.receive_message)
+        self.task_consumer.consume()
+
+        debug('Ready to accept tasks!')
+
+        while self._state != CLOSE and self.connection:
+            self.maybe_shutdown()
+            if self.qos.prev != self.qos.value:     # pragma: no cover
+                self.qos.update()
+            try:
+                self.connection.drain_events(timeout=10.0)
+            except socket.timeout:
+                pass
+            except socket.error:
+                if self._state != CLOSE:            # pragma: no cover
+                    raise
