@@ -5,24 +5,23 @@
 
     Remote control commands.
 
-    :copyright: (c) 2009 - 2012 by Ask Solem.
-    :license: BSD, see LICENSE for more details.
-
 """
 from __future__ import absolute_import
 
 from datetime import datetime
 
-from ..platforms import signals as _signals
-from ..registry import tasks
-from ..utils import timeutils
-from ..utils.compat import UserDict
-from ..utils.encoding import safe_repr
+from kombu.utils.encoding import safe_repr
+
+from celery.platforms import signals as _signals
+from celery.utils import timeutils
+from celery.utils.compat import UserDict
+from celery.utils.log import get_logger
 
 from . import state
 from .state import revoked
 
-TASK_INFO_FIELDS = ("exchange", "routing_key", "rate_limit")
+TASK_INFO_FIELDS = ('exchange', 'routing_key', 'rate_limit')
+logger = get_logger(__name__)
 
 
 class Panel(UserDict):
@@ -38,17 +37,22 @@ class Panel(UserDict):
 def revoke(panel, task_id, terminate=False, signal=None, **kwargs):
     """Revoke task by task id."""
     revoked.add(task_id)
-    action = "revoked"
+    action = 'revoked'
     if terminate:
-        signum = _signals.signum(signal or "TERM")
+        signum = _signals.signum(signal or 'TERM')
         for request in state.active_requests:
-            if request.task_id == task_id:
-                action = "terminated (%s)" % (signum, )
+            if request.id == task_id:
+                action = 'terminated (%s)' % (signum, )
                 request.terminate(panel.consumer.pool, signal=signum)
                 break
 
-    panel.logger.info("Task %s %s.", task_id, action)
-    return {"ok": "task %s %s" % (task_id, action)}
+    logger.info('Task %s %s.', task_id, action)
+    return {'ok': 'task %s %s' % (task_id, action)}
+
+
+@Panel.register
+def report(panel):
+    return {'ok': panel.app.bugreport()}
 
 
 @Panel.register
@@ -56,28 +60,28 @@ def enable_events(panel):
     dispatcher = panel.consumer.event_dispatcher
     if not dispatcher.enabled:
         dispatcher.enable()
-        dispatcher.send("worker-online")
-        panel.logger.info("Events enabled by remote.")
-        return {"ok": "events enabled"}
-    return {"ok": "events already enabled"}
+        dispatcher.send('worker-online')
+        logger.info('Events enabled by remote.')
+        return {'ok': 'events enabled'}
+    return {'ok': 'events already enabled'}
 
 
 @Panel.register
 def disable_events(panel):
     dispatcher = panel.consumer.event_dispatcher
     if dispatcher.enabled:
-        dispatcher.send("worker-offline")
+        dispatcher.send('worker-offline')
         dispatcher.disable()
-        panel.logger.info("Events disabled by remote.")
-        return {"ok": "events disabled"}
-    return {"ok": "events already disabled"}
+        logger.info('Events disabled by remote.')
+        return {'ok': 'events disabled'}
+    return {'ok': 'events already disabled'}
 
 
 @Panel.register
 def heartbeat(panel):
-    panel.logger.debug("Heartbeat requested by remote.")
+    logger.debug('Heartbeat requested by remote.')
     dispatcher = panel.consumer.event_dispatcher
-    dispatcher.send("worker-heartbeat", **state.SOFTWARE_INFO)
+    dispatcher.send('worker-heartbeat', freq=5, **state.SOFTWARE_INFO)
 
 
 @Panel.register
@@ -94,67 +98,66 @@ def rate_limit(panel, task_name, rate_limit, **kwargs):
     try:
         timeutils.rate(rate_limit)
     except ValueError, exc:
-        return {"error": "Invalid rate limit string: %s" % exc}
+        return {'error': 'Invalid rate limit string: %s' % exc}
 
     try:
-        tasks[task_name].rate_limit = rate_limit
+        panel.app.tasks[task_name].rate_limit = rate_limit
     except KeyError:
-        panel.logger.error("Rate limit attempt for unknown task %s",
-                           task_name, exc_info=True)
-        return {"error": "unknown task"}
+        logger.error('Rate limit attempt for unknown task %s',
+                     task_name, exc_info=True)
+        return {'error': 'unknown task'}
 
-    if not hasattr(panel.consumer.ready_queue, "refresh"):
-        panel.logger.error("Rate limit attempt, but rate limits disabled.")
-        return {"error": "rate limits disabled"}
+    if not hasattr(panel.consumer.ready_queue, 'refresh'):
+        logger.error('Rate limit attempt, but rate limits disabled.')
+        return {'error': 'rate limits disabled'}
 
     panel.consumer.ready_queue.refresh()
 
     if not rate_limit:
-        panel.logger.info("Disabled rate limits for tasks of type %s",
-                          task_name)
-        return {"ok": "rate limit disabled successfully"}
+        logger.info('Rate limits disabled for tasks of type %s', task_name)
+        return {'ok': 'rate limit disabled successfully'}
 
-    panel.logger.info("New rate limit for tasks of type %s: %s.",
-                      task_name, rate_limit)
-    return {"ok": "new rate limit set successfully"}
+    logger.info('New rate limit for tasks of type %s: %s.',
+                task_name, rate_limit)
+    return {'ok': 'new rate limit set successfully'}
 
 
 @Panel.register
 def time_limit(panel, task_name=None, hard=None, soft=None, **kwargs):
     try:
-        task = tasks[task_name]
+        task = panel.app.tasks[task_name]
     except KeyError:
-        panel.logger.error("Change time limit attempt for unknown task %s",
-                           task_name, exc_info=True)
-        return {"error": "unknown task"}
+        logger.error('Change time limit attempt for unknown task %s',
+                     task_name, exc_info=True)
+        return {'error': 'unknown task'}
 
     task.soft_time_limit = soft
     task.time_limit = hard
 
-    panel.logger.info("New time limits for tasks of type %s: soft=%s hard=%s",
-                      task_name, soft, hard)
-    return {"ok": "time limits set successfully"}
+    logger.info('New time limits for tasks of type %s: soft=%s hard=%s',
+                task_name, soft, hard)
+    return {'ok': 'time limits set successfully'}
 
 
 @Panel.register
 def dump_schedule(panel, safe=False, **kwargs):
-    schedule = panel.consumer.eta_schedule.schedule
+    schedule = panel.consumer.timer.schedule
     if not schedule.queue:
-        panel.logger.info("--Empty schedule--")
+        logger.debug('--Empty schedule--')
         return []
 
-    formatitem = lambda (i, item): "%s. %s pri%s %r" % (i,
-            datetime.utcfromtimestamp(item["eta"]),
-            item["priority"],
-            item["item"])
+    formatitem = lambda (i, item): '%s. %s pri%s %r' % (i,
+            datetime.utcfromtimestamp(item['eta']),
+            item['priority'],
+            item['item'])
     info = map(formatitem, enumerate(schedule.info()))
-    panel.logger.debug("* Dump of current schedule:\n%s", "\n".join(info))
+    logger.debug('* Dump of current schedule:\n%s', '\n'.join(info))
     scheduled_tasks = []
     for item in schedule.info():
-        scheduled_tasks.append({"eta": item["eta"],
-                                "priority": item["priority"],
-                                "request":
-                                    item["item"].args[0].info(safe=safe)})
+        scheduled_tasks.append({'eta': item['eta'],
+                                'priority': item['priority'],
+                                'request':
+                                    item['item'].args[0].info(safe=safe)})
     return scheduled_tasks
 
 
@@ -163,10 +166,10 @@ def dump_reserved(panel, safe=False, **kwargs):
     ready_queue = panel.consumer.ready_queue
     reserved = ready_queue.items
     if not reserved:
-        panel.logger.info("--Empty queue--")
+        logger.debug('--Empty queue--')
         return []
-    panel.logger.debug("* Dump of currently reserved tasks:\n%s",
-                       "\n".join(map(safe_repr, reserved)))
+    logger.debug('* Dump of currently reserved tasks:\n%s',
+                 '\n'.join(map(safe_repr, reserved)))
     return [request.info(safe=safe)
             for request in reserved]
 
@@ -182,10 +185,10 @@ def stats(panel, **kwargs):
     asinfo = {}
     if panel.consumer.controller.autoscaler:
         asinfo = panel.consumer.controller.autoscaler.info()
-    return {"total": state.total_count,
-            "consumer": panel.consumer.info,
-            "pool": panel.consumer.pool.info,
-            "autoscaler": asinfo}
+    return {'total': state.total_count,
+            'consumer': panel.consumer.info,
+            'pool': panel.consumer.pool.info,
+            'autoscaler': asinfo}
 
 
 @Panel.register
@@ -195,27 +198,27 @@ def dump_revoked(panel, **kwargs):
 
 @Panel.register
 def dump_tasks(panel, **kwargs):
+    tasks = panel.app.tasks
 
     def _extract_info(task):
         fields = dict((field, str(getattr(task, field, None)))
                         for field in TASK_INFO_FIELDS
                             if getattr(task, field, None) is not None)
-        info = map("=".join, fields.items())
+        info = map('='.join, fields.items())
         if not info:
             return task.name
-        return "%s [%s]" % (task.name, " ".join(info))
+        return '%s [%s]' % (task.name, ' '.join(info))
 
     info = map(_extract_info, (tasks[task]
-                                        for task in sorted(tasks.keys())))
-    panel.logger.debug("* Dump of currently registered tasks:\n%s",
-                       "\n".join(info))
+                                    for task in sorted(tasks.keys())))
+    logger.debug('* Dump of currently registered tasks:\n%s', '\n'.join(info))
 
     return info
 
 
 @Panel.register
 def ping(panel, **kwargs):
-    return "pong"
+    return 'pong'
 
 
 @Panel.register
@@ -224,7 +227,7 @@ def pool_grow(panel, n=1, **kwargs):
         panel.consumer.controller.autoscaler.force_scale_up(n)
     else:
         panel.consumer.pool.grow(n)
-    return {"ok": "spawned worker processes"}
+    return {'ok': 'spawned worker processes'}
 
 
 @Panel.register
@@ -233,13 +236,13 @@ def pool_shrink(panel, n=1, **kwargs):
         panel.consumer.controller.autoscaler.force_scale_down(n)
     else:
         panel.consumer.pool.shrink(n)
-    return {"ok": "terminated worker processes"}
+    return {'ok': 'terminated worker processes'}
 
 
 @Panel.register
 def pool_restart(panel, modules=None, reload=False, reloader=None, **kwargs):
     panel.consumer.controller.reload(modules, reload, reloader=reloader)
-    return {"ok": "reload started"}
+    return {'ok': 'reload started'}
 
 
 @Panel.register
@@ -247,39 +250,42 @@ def autoscale(panel, max=None, min=None):
     autoscaler = panel.consumer.controller.autoscaler
     if autoscaler:
         max_, min_ = autoscaler.update(max, min)
-        return {"ok": "autoscale now min=%r max=%r" % (max_, min_)}
-    raise ValueError("Autoscale not enabled")
+        return {'ok': 'autoscale now min=%r max=%r' % (max_, min_)}
+    raise ValueError('Autoscale not enabled')
 
 
 @Panel.register
-def shutdown(panel, **kwargs):
-    panel.logger.warning("Got shutdown from remote.")
-    raise SystemExit("Got shutdown from remote")
+def shutdown(panel, msg='Got shutdown from remote', **kwargs):
+    logger.warning(msg)
+    raise SystemExit(msg)
 
 
 @Panel.register
-def add_consumer(panel, queue=None, exchange=None, exchange_type="direct",
+def add_consumer(panel, queue, exchange=None, exchange_type=None,
         routing_key=None, **options):
     cset = panel.consumer.task_consumer
+    exchange = queue if exchange is None else exchange
+    routing_key = queue if routing_key is None else routing_key
+    exchange_type = 'direct' if exchange_type is None else exchange_type
     if not cset.consuming_from(queue):
-        declaration = dict(queue=queue,
-                           exchange=exchange,
-                           exchange_type=exchange_type,
-                           routing_key=routing_key,
-                           **options)
-        cset.add_consumer_from_dict(**declaration)
+        q = panel.app.amqp.queues.add(queue,
+                exchange=exchange,
+                exchange_type=exchange_type,
+                routing_key=routing_key, **options)
+        cset.add_queue(q)
         cset.consume()
-        panel.logger.info("Started consuming from %r", declaration)
-        return {"ok": "started consuming from %s" % (queue, )}
+        logger.info('Started consuming from %r', queue)
+        return {'ok': 'started consuming from %r' % (queue, )}
     else:
-        return {"ok": "already consuming from %s" % (queue, )}
+        return {'ok': 'already consuming from %r' % (queue, )}
 
 
 @Panel.register
 def cancel_consumer(panel, queue=None, **_):
+    panel.app.amqp.queues.select_remove(queue)
     cset = panel.consumer.task_consumer
     cset.cancel_by_queue(queue)
-    return {"ok": "no longer consuming from %s" % (queue, )}
+    return {'ok': 'no longer consuming from %s' % (queue, )}
 
 
 @Panel.register
