@@ -12,9 +12,6 @@
     rate limits will also disable this machinery,
     and can improve performance.
 
-    :copyright: (c) 2009 - 2012 by Ask Solem.
-    :license: BSD, see LICENSE for more details.
-
 """
 from __future__ import absolute_import
 
@@ -22,29 +19,33 @@ import logging
 
 from Queue import Empty
 
-from ..abstract import StartStopComponent
-from ..app import app_or_default
-from ..utils.threads import bgThread
+from celery.app import app_or_default
+from celery.utils.threads import bgThread
+from celery.utils.log import get_logger
+
+from .bootsteps import StartStopComponent
+
+logger = get_logger(__name__)
 
 
 class WorkerComponent(StartStopComponent):
-    name = "worker.mediator"
-    requires = ("pool", "queues", )
+    name = 'worker.mediator'
+    requires = ('pool', 'queues', )
 
     def __init__(self, w, **kwargs):
         w.mediator = None
 
     def include_if(self, w):
-        return not w.disable_rate_limits or w.pool_cls.requires_mediator
+        return w.start_mediator
 
     def create(self, w):
         m = w.mediator = self.instantiate(w.mediator_cls, w.ready_queue,
-                                          app=w.app, callback=w.process_task,
-                                          logger=w.logger)
+                                          app=w.app, callback=w.process_task)
         return m
 
 
 class Mediator(bgThread):
+    """Mediator thread."""
 
     #: The task queue, a :class:`~Queue.Queue` instance.
     ready_queue = None
@@ -52,12 +53,11 @@ class Mediator(bgThread):
     #: Callback called when a task is obtained.
     callback = None
 
-    def __init__(self, ready_queue, callback, logger=None, app=None):
+    def __init__(self, ready_queue, callback, app=None, **kw):
         self.app = app_or_default(app)
-        self.logger = logger or self.app.log.get_default_logger()
         self.ready_queue = ready_queue
         self.callback = callback
-        self._does_debug = self.logger.isEnabledFor(logging.DEBUG)
+        self._does_debug = logger.isEnabledFor(logging.DEBUG)
         super(Mediator, self).__init__()
 
     def body(self):
@@ -70,16 +70,14 @@ class Mediator(bgThread):
             return
 
         if self._does_debug:
-            self.logger.debug(
-                "Mediator: Running callback for task: %s[%s]" % (
-                    task.task_name, task.task_id))
+            logger.debug('Mediator: Running callback for task: %s[%s]',
+                         task.name, task.id)
 
         try:
             self.callback(task)
         except Exception, exc:
-            self.logger.error("Mediator callback raised exception %r",
-                              exc, exc_info=True,
-                              extra={"data": {"id": task.task_id,
-                                              "name": task.task_name,
-                                              "hostname": task.hostname}})
-    move = body   # XXX compat
+            logger.error('Mediator callback raised exception %r',
+                         exc, exc_info=True,
+                         extra={'data': {'id': task.id,
+                                         'name': task.name,
+                                         'hostname': task.hostname}})
